rm(list=ls())
##Set the working directory manually or with the following commands...
#getwd()
#setwd("pathway")

## read and attach the data
anova_data <- read.table("ANOVA_data.txt", header=T, row.names="ID")
attach(anova_data)

## use the factor() function to tag GROUP as a categorical variable and draw a boxplot for each group
factor(GROUP)
boxplot(FLOWERING_PERIOD ~ GROUP)

### PART I: ANOVA with built-in R functions ###

## use the lm() and anova() functions to execute the ANOVA
anova_model <- lm(FLOWERING_PERIOD ~ GROUP)
anova(anova_model)
tapply(FLOWERING_PERIOD,GROUP,mean)

### PART II: ANOVA by partitioning the sum of squares ###

## get the subset of the data for each group
Y_group1 <- FLOWERING_PERIOD[GROUP=="Unmanipulated"]
Y_group2 <- FLOWERING_PERIOD[GROUP=="Control"]
Y_group3 <- FLOWERING_PERIOD[GROUP=="Treatment"]

## get the number of groups (a) and the number of replicates per group (n)
a <- length(levels(GROUP))
n <- length(Y_group1)

## calculate the grand mean and the group means
grand_mean <- mean(FLOWERING_PERIOD)
group1_mean <- mean(Y_group1)
group2_mean <- mean(Y_group2)
group3_mean <- mean(Y_group3)

## calculate the sum of squares (SS) WITHIN the groups
SS_within1 = sum((Y_group1 - group1_mean)^2)
SS_within2 = sum((Y_group2 - group2_mean)^2)
SS_within3 = sum((Y_group3 - group3_mean)^2)
SS_within = SS_within1 + SS_within2 + SS_within3

## calculate the sum of squares (SS) AMONG the groups
SS_among1 = n * sum((group1_mean - grand_mean)^2)
SS_among2 = n * sum((group2_mean - grand_mean)^2)
SS_among3 = n * sum((group3_mean - grand_mean)^2)
SS_among = SS_among1 + SS_among2 + SS_among3

## calculate the mean square (MS) values
df_among = (a-1)
df_within = a*(n-1)
MS_among = SS_among/df_among
MS_within = SS_within/df_within

## calculate the F-ratio and corresponding P-value
F_ratio = MS_among/MS_within
P_value = pf(F_ratio, df_among, df_within, lower.tail=F)

tab <- array(0,c(2,4))
colnames(tab) <- c(" Mean Sq", "df", "F value", "P")
rownames(tab) <- c("Group","Residuals")
tab[1,] <- round(c(MS_among,df_among,F_ratio, P_value),4)
MS_within <- round(MS_within,4)
tab[2,] <- c(MS_within,df_within, "/", "/") 
tab

## PART III: ANOVA using a "dummy variable" regression ###

anova_model <- lm(FLOWERING_PERIOD ~ GROUP)
summary(anova_model)
anova_data$new <- rep(0,length(GROUP))
anova_data$new[GROUP=="Control"]  <- 1
anova_data$new[GROUP=="Treatment"] <- 2
anova_model1 <- lm(FLOWERING_PERIOD ~ factor(anova_data$new))
summary(anova_model1)

## add "dummy variables" X1 and X2 to the data frame and initialize them to 0
anova_data$X1 <- numeric(length(GROUP))
anova_data$X2 <- numeric(length(GROUP))

## for the Unmanipulated group, X1=0 and X2=0; for the Control group, X1=1
anova_data$X1[GROUP=="Control"] <- 1

## for the Treatment group, X2=1
anova_data$X2[GROUP=="Treatment"] <- 1

## now perform multiple regression with the dummy variables
regression_model <- lm(FLOWERING_PERIOD ~ anova_data$X1 + anova_data$X2)
summary(regression_model)

## to perform an ANOVA for the regression, we first need the fitted values
coeffs <- regression_model$coefficients
Y_hats <- coeffs[1] + coeffs[2]*anova_data$X1 + coeffs[3]*anova_data$X2

## calculate the SS for the regression (SSreg) and the residuals (SSresid)
SSreg <- sum((Y_hats - grand_mean)^2)
SSresid <- sum((FLOWERING_PERIOD - Y_hats)^2)

## finally, calculate the F-ratio and P-value for the regression
F_ratio <- (SSreg/df_among)/(SSresid/df_within)
P_value <- pf(F_ratio, df_among, df_within, lower.tail=F)


### PART IV: Analysis using OpenBUGS (means format)###

x <- c(1,1,1,1,2,2,2,2,3,3,3,3)
n_obs <- length(x)
z <- FLOWERING_PERIOD

## Call package  

library(R2OpenBUGS)

### Fitting the model
# Write model
Anovam<-function()


## Priors
{

 for (i in 1:3)
   {
     a[i] ~ dnorm(0.0,1.0E-6)
   }
 tau ~ dgamma(0.001,0.001)

## Likelihood
 for (i in 1:n)
   {
   mean[i] <- a[x[i]]
   Y[i]  ~ dnorm(mean[i],tau)
   
   }
  d21 <- a[2]-a[1]
  d31 <- a[3]-a[1]
  d32 <- a[3]-a[2]

  }

write.model(Anovam, "Anovam.txt")

# Bundle data
win.data <- list(Y=z,x=x, n=n_obs)

# Inits function

inits <- function() 
   list(a=runif(3,1,10),tau=runif(1))

# Parameters to estimate
params <- c("a","d21","d31","d32")

# MCMC settings
nc = 3 
ni=10000
nb=1000
nt=10

# Start Gibbs sampler
out <- bugs(data = win.data, inits = inits, parameters = params, model = "Anovam.txt",
n.thin = nt, n.chains = nc, n.burnin = nb, n.iter = ni, digits=5, codaPkg=T)

library(coda)
reg.coda<-read.bugs(out)
results<-summary(reg.coda)
results


### PART V: Analysis using OpenBUGS (regression format using a reference class)##


### Fitting the model
# Write model
Anovar<-function()

## Priors
{
 
 reference ~ dnorm(0,1.0E-6) 
 for (i in 2:3)
   {
     d[i] ~ dnorm(0.0,1.0E-6)
   }
 d[1] <- 0
 tau ~ dgamma(0.001,0.001)

## Likelihood
 for (i in 1:n)
   {
   mean[i] <- reference + d[x[i]]
   Y[i]  ~ dnorm(mean[i],tau)
   
   }

  }

write.model(Anovar, "Anovar.txt")

# Bundle data
win.data <- list(Y=z,x=x, n=n_obs)

# Inits function

inits <- function() 
   list(reference= runif(1,0,30), d= c(NA,0,0), tau=runif(1))

# Parameters to estimate
params <- c("reference","d")

# MCMC settings
nc = 3 
ni=10000
nb=1000
nt=10

# Start Gibbs sampler
out <- bugs(data = win.data, inits = inits, parameters = params, model = "Anovar.txt",
n.thin = nt, n.chains = nc, n.burnin = nb, n.iter = ni,  digits=5, codaPkg=T)
reg.coda<-read.bugs(out)
results<-summary(reg.coda)
results
## detach the data
detach(anova_data)